"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DistributedPool = void 0;
const grpc_1 = __importDefault(require("grpc"));
const worker_pb_1 = __importDefault(require("../gen/worker_pb"));
const worker_grpc_pb_1 = __importDefault(require("../gen/worker_grpc_pb"));
const queue_1 = require("./queue");
const DEFAULT_WORKERS = 3;
const isHostPort = (arg) => {
    return arg && typeof arg.host === 'string' && typeof arg.port === 'string';
};
const isHostPortArray = (arg) => {
    if (arg && arg.length !== undefined) {
        if (arg.length > 0)
            return isHostPort(arg[0]);
        return true;
    }
    return false;
};
class DistributedPool {
    constructor(nodes) {
        this.nodes = new queue_1.Queue();
        let uris;
        if (nodes.length === 0)
            throw Error('Node URIs must be provided');
        if (isHostPortArray(nodes)) {
            uris = nodes.map(({ host, port }) => `${host}:${port}`);
        }
        else {
            uris = nodes;
        }
        for (let uri of uris) {
            this.nodes.add({
                availableWorkers: DEFAULT_WORKERS,
                client: new worker_grpc_pb_1.default.WorkerClient(uri, grpc_1.default.credentials.createInsecure()),
            });
        }
    }
    addJob(fn, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const node = this.nodes.pop();
            const request = new worker_pb_1.default.AddJobRequest();
            request.setFn(fn.toString());
            request.setArgsList(args);
            return new Promise((resolve, reject) => {
                if (node === null || node.availableWorkers === 0) {
                    reject(Error('No worker nodes available'));
                    return;
                }
                node.availableWorkers--;
                this.nodes.add(node);
                node.client.addJob(request, function (error, response) {
                    node.availableWorkers++;
                    if (error) {
                        reject(error);
                        return;
                    }
                    resolve(response.getResult());
                });
            });
        });
    }
}
exports.DistributedPool = DistributedPool;
exports.default = DistributedPool;
//# sourceMappingURL=distributedPool.js.map